@@ -193,6 +193,7 @@ class Purchase(Workflow, ModelSQL, ModelView, TaxableMixin):
     @classmethod
     def __setup__(cls):
         super(Purchase, cls).__setup__()
+        cls.create_date.select = True
         cls._order = [
             ('purchase_date', 'DESC'),
             ('id', 'DESC'),
@@ -316,9 +317,6 @@ class Purchase(Workflow, ModelSQL, ModelView, TaxableMixin):
                     values=['processing'],
                     where=sql_table.id.in_(sub_query.select(sub_query.id))))
 
-        # Add index on create_date
-        table.index_action('create_date', action='add')
-
     @classmethod
     def default_payment_term(cls):
         PaymentTerm = Pool().get('account.invoice.payment_term')
@@ -1050,9 +1048,11 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
     delivery_date = fields.Function(fields.Date('Delivery Date',
             states={
                 'invisible': ((Eval('type') != 'line')
-                    | Eval('delivery_date_edit', False)),
+                    | (Eval('delivery_date_edit', False)
+                        & ~Eval('purchase_state').in_(
+                            ['processing', 'done', 'cancelled']))),
                 },
-            depends=['type', 'delivery_date_edit']),
+            depends=['type', 'delivery_date_edit', 'purchase_state']),
         'on_change_with_delivery_date')
     delivery_date_edit = fields.Boolean(
         "Edit Delivery Date",
@@ -1067,7 +1067,9 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
         "Delivery Date",
         states={
             'invisible': ((Eval('type') != 'line')
-                | ~Eval('delivery_date_edit', False)),
+                | ~Eval('delivery_date_edit', False)
+                | Eval('purchase_state').in_(
+                    ['processing', 'done', 'cancelled'])),
             'readonly': Eval('purchase_state').in_([
                     'processing', 'done', 'cancel']),
             },
@@ -1292,7 +1294,8 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
 
     def get_from_location(self, name):
         if (self.quantity or 0) >= 0:
-            return self.purchase.party.supplier_location.id
+            if self.purchase.party.supplier_location:
+                return self.purchase.party.supplier_location.id
         elif self.purchase.return_from_location:
             return self.purchase.return_from_location.id
 
@@ -1300,7 +1303,7 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
         if (self.quantity or 0) >= 0:
             if self.purchase.warehouse:
                 return self.purchase.warehouse.input_location.id
-        else:
+        elif self.purchase.party.supplier_location:
             return self.purchase.party.supplier_location.id
 
     @fields.depends('product_supplier', 'quantity', 'moves', 'purchase',
@@ -1310,10 +1313,10 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
     def on_change_with_delivery_date(self, name=None):
         pool = Pool()
         Date = pool.get('ir.date')
-        if self.moves:
+        moves = [m for m in self.moves if m.state != 'cancel']
+        if moves:
             dates = filter(
-                None, (m.effective_date or m.planned_date for m in self.moves
-                    if m.state != 'cancel'))
+                None, (m.effective_date or m.planned_date for m in moves))
             return min(dates, default=None)
         delivery_date = None
         if self.delivery_date_edit:
@@ -1347,6 +1350,8 @@ class PurchaseLine(sequence_ordered(), ModelSQL, ModelView):
 
         invoice_line = InvoiceLine()
         invoice_line.type = self.type
+        invoice_line.currency = self.purchase.currency
+        invoice_line.company = self.purchase.company
         invoice_line.description = self.description
         invoice_line.note = self.note
         invoice_line.origin = self
